#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>

#define SIZECACHE 100
#define MAXWORD 100

char cache[SIZECACHE]; /* tampon pour writeChar */
int pcache = 0; /* prochaine position
                        libre dans cache */

struct node {
    char *word;
    int cpt;
    struct node *left;
    struct node *right;
};


/* readWord : lit le mot ou le caractere suivant
    sur l'entree */
int readWord (char *word, int lim)
{
    int c, readChar(void);
    void writeChar(int);
    char *m = word;
    while (isspace(c = readChar()))
        ;
    if (c != EOF)
        *m++ = c;
    if (!isalpha(c)) {
        *m = '\0';
        return c;
    }

    for (; --lim > 0;  m++)
        if (!isalnum(*m = readChar())) {
            writeChar(*m);
            break;
        }
    *m = '\0';
    return word[0];

}

/* lit un caractere (eventuellement remis sur l'entree) */

int readChar(void)
{
    return (pcache > 0) ? cache[--pcache] : getchar();
}

/* remet c sur l'entree */
void writeChar(int c)
{
    if (pcache >= SIZECACHE)
        printf("writeChar : trop de caractere\n");
    else
        cache[pcache++] = c;
}

/* allocNode cree un noeud */
struct node *allocNode(void) {
    return (struct node *) malloc (sizeof(struct node));
}

/* copyString : copie la chaine de caractere donnee en argument
 dans un endroit sur obtenu par un appel a malloc */
char *copyString(char *s)
{
    char *p;
    p = (char *) malloc(strlen(s)+1); /* +1 pour '\0' */
    if (p != NULL)
        strcpy (p, s);
    return p;

}

/* addTree : ajoute un noeud contenant la chaine m au niveau de p ou en dessous */
struct node *addTree(struct node *p, char *m) {
    int cond;
    if (p == NULL) {
        p = allocNode();
        p->word = copyString(m);
        p->cpt = 1;
        p->left = p->right = NULL;
    } else if ((cond = strcmp(m, p->word)) == 0) {
        p->cpt++;
    } else if (cond < 0) {
        /* mot dans le sous arbre gauche */
        p->left = addTree(p->left, m);
    } else
        /* mot dans le sous arbre droit */
        p->right = addTree(p->right, m);
    return p;
}

/* Affiche l'arbre passe en parametre */
void displayTree(struct node *p)
{
    if (p != NULL) {
        displayTree(p->left);
        printf("%4d %s\n", p->cpt, p->word);
        displayTree(p->right);
    }
}

int main()
{
    struct node *root;
    char word[MAXWORD];

    root = NULL;
    while (readWord(word, MAXWORD) != EOF)
        if (isalpha(word[0]))
            root = addTree(root, word);
    displayTree(root);
    return 0;
}




